package decorator;

public class Main {

    public static void main(String[] args) {

        // São instanciados os sensores de pressão, temperatura e nível da água
        Sensor pressure = new Sensor();
        Sensor temperature = new Sensor();
        Sensor waterLevel = new Sensor();

        // O monitoramento do exemplo será composto da seguinte condição:
        // (pressure > 60 && waterLevel < 40) || (temperature > 50 || temperature < 10)

        // Para cada operação lógica da condição, um monitoramento simples é criado.
        GreaterThanMonitoring pressureOver60 = new GreaterThanMonitoring(pressure, 60);
        LessThanMonitoring waterLevelBelow40 = new LessThanMonitoring(waterLevel, 40);
        GreaterThanMonitoring temperatureOver50 = new GreaterThanMonitoring(temperature, 50);
        LessThanMonitoring temperatureBelow10 = new LessThanMonitoring(temperature, 10);

        // Dois decorators são criados para encapsular os monitoramentos simples em operações binárias
        AndDecorator pressureAndWaterLevelMonitoring = new AndDecorator(pressureOver60, waterLevelBelow40);
        OrDecorator temperatureRangeMonitoring = new OrDecorator(temperatureOver50, temperatureBelow10);

        // Um decorator é criado para encapsular os dois decorators criados em uma única operação binária
        OrDecorator mainMonitoring = new OrDecorator(pressureAndWaterLevelMonitoring, temperatureRangeMonitoring);

        // Sensores identificam os valores:
        pressure.setValue(50);
        waterLevel.setValue(45);
        temperature.setValue(49);

        if(mainMonitoring.evaluateCondition()) {
            System.out.println("Uma falha foi detectada!");
        } else {
            System.out.println("Nenhuma falha foi detectada.");
        }

    }
}
